from dbus.exceptions import DBusException
from upstart.system import UpstartSystem, DirectUpstartBus
from upstart.job import UpstartJob

from jadi import component
from aj.plugins.services.api import ServiceManager, Service, ServiceOperationError


@component(ServiceManager)
class UpstartServiceManager(ServiceManager):
    id = 'upstart'
    name = 'Upstart'

    @classmethod
    def __verify__(cls):
        try:
            UpstartSystem()
            return True
        except:
            try:
                UpstartSystem(bus=DirectUpstartBus())
                return True
            except:
                return False

    def __init__(self, context):
        self.bus = None
        try:
            self.upstart = UpstartSystem()
        except:
            self.bus = DirectUpstartBus()
            self.upstart = UpstartSystem(bus=self.bus)

    def __fix_name(self, name):
        return name.replace('_2d', '-').replace('_2e', '.')

    def list(self):
        for job_name in self.upstart.get_all_jobs():
            yield self.get_service(job_name)

    def get_service(self, _id):
        job = UpstartJob(_id, bus=self.bus)
        svc = Service(self)
        svc.id = _id
        svc.name = self.__fix_name(_id)
        try:
            svc.state = job.get_status()['state']
            svc.running = svc.state == 'running'
        except:
            svc.running = False
        return svc

    def start(self, _id):
        try:
            UpstartJob(_id).start()
        except DBusException as e:
            raise ServiceOperationError(e)

    def stop(self, _id):
        try:
            UpstartJob(_id).stop()
        except DBusException as e:
            raise ServiceOperationError(e)

    def restart(self, _id):
        try:
            UpstartJob(_id).restart()
        except DBusException as e:
            raise ServiceOperationError(e)



from jadi import component
from aj.plugins.dashboard.api import Widget
from aj.plugins.services.api import ServiceManager


@component(Widget)
class ServiceWidget(Widget):
    id = 'service'
    name = 'Service'
    template = '/services:resources/partial/widget.html'
    config_template = '/services:resources/partial/widget.config.html'

    def __init__(self, context):
        Widget.__init__(self, context)

    def get_value(self, config):
        manager_id = config.get('manager_id', None)
        service_id = config.get('service_id', None)
        if not manager_id or not service_id:
            return None
        for mgr in ServiceManager.all(self.context):
            if mgr.id == manager_id:
                svc = mgr.get_service(service_id)
                return {
                    'id': svc.id,
                    'name': svc.name,
                    'managerId': svc.manager.id,
                    'state': svc.state,
                    'isRunning': svc.running,
                }



import os
import subprocess

from jadi import component
from aj.plugins.services.api import ServiceManager, Service


INIT_D = '/etc/init.d'
UPSTART_PATTERN = '/etc/init/%s.conf'


@component(ServiceManager)
class SysVServiceManager(ServiceManager):
    id = 'sysv'
    name = 'System V'

    @classmethod
    def __verify__(cls):
        return os.path.exists(INIT_D)

    def __init__(self, context):
        pass

    def list(self):
        for _id in os.listdir(INIT_D):
            path = os.path.join(INIT_D, _id)
            if _id.startswith('.'):
                continue
            if _id.startswith('rc'):
                continue
            if os.path.islink(path):
                continue
            if os.path.exists(UPSTART_PATTERN % _id):
                continue
            yield self.get_service(_id)

    def get_service(self, _id):
        svc = Service(self)
        svc.id = svc.name = _id
        try:
            svc.running = self._run_action(_id, 'status')
            svc.state = 'running' if svc.running else 'stopped'
        except:
            svc.running = False
        return svc

    def _run_action(self, _id, action):
        return subprocess.call([os.path.join(INIT_D, _id), action], close_fds=True) == 0

    def start(self, _id):
        self._run_action(_id, 'start')

    def stop(self, _id):
        self._run_action(_id, 'stop')

    def restart(self, _id):
        self._run_action(_id, 'restart')



from jadi import component

from aj.plugins.core.api.sidebar import SidebarItemProvider

from .api import ServiceManager


@component(SidebarItemProvider)
class ItemProvider(SidebarItemProvider):
    def __init__(self, context):
        self.context = context

    def provide(self):
        children = [{
            'attach': 'services',
            'name': mgr.name,
            'icon': 'cog',
            'url': '/view/services/%s' % mgr.id,
            'children': [],
        } for mgr in ServiceManager.all(self.context)]

        return [
            {
                'attach': 'category:software',
                'name': 'Services',
                'icon': 'cogs',
                'url': '/view/services',
                'children': children
            }
        ]



from jadi import component
from aj.api.http import url, HttpPlugin

from aj.api.endpoint import endpoint, EndpointError
from aj.plugins.services.api import ServiceManager, ServiceOperationError


@component(HttpPlugin)
class Handler(HttpPlugin):
    def __init__(self, context):
        self.context = context
        self.managers = dict((x.id, x) for x in ServiceManager.all(self.context))

    def __service_to_json(self, svc):
        return {
            'id': svc.id,
            'name': svc.name,
            'state': svc.state,
            'running': svc.running,
            'managerId': svc.manager.id,
        }

    @url(r'/api/services/managers')
    @endpoint(api=True)
    def handle_api_managers(self, http_context):
        return [
            {
                'id': mgr.id,
                'name': mgr.name,
            } for mgr in self.managers.values()
        ]

    @url(r'/api/services/list/(?P<manager_id>\w+)')
    @endpoint(api=True)
    def handle_api_list(self, http_context, manager_id=None):
        return [self.__service_to_json(svc) for svc in self.managers[manager_id].list()]

    @url(r'/api/services/get/(?P<manager_id>\w+)/(?P<service_id>.+)')
    @endpoint(api=True)
    def handle_api_get(self, http_context, manager_id=None, service_id=None):
        return self.__service_to_json(self.managers[manager_id].get_service(service_id))

    @url(r'/api/services/do/(?P<operation>\w+)/(?P<manager_id>\w+)/(?P<service_id>.+)')
    @endpoint(api=True)
    def handle_api_operate(self, http_context, manager_id=None, operation=None, service_id=None):
        if operation not in ['start', 'stop', 'restart']:
            return
        try:
            getattr(self.managers[manager_id], operation)(service_id)
        except ServiceOperationError as e:
            raise EndpointError(e)



import api
import managers.systemd_manager

try:
    import managers.upstart_manager
except ImportError:
    pass

import managers.sysv_manager
import main
import views

def init(plugin_manager):
    if 'dashboard' in plugin_manager and plugin_manager['dashboard']['imported']:
        import widget
